// Datei: dl1416.C
//
// Ansteuerung DL1416 LED-Anzeige
//
// Pinbelegung:
//  1 D5       D6 20
//  2 D4      GND 19
//  3 D0      VCC 18
//  4 D1       nc 17
//  5 D2       nc 16
//  6 D3       nc 15
//  7 /CE      nc 14
//  8 /WR      nc 13
//  9 /CU      nc 12
// 10 A0       A1 11
//
// D0..D6 die Datenleitungen halt.
// /CE,/WR arbeiten wie bei jedem anderen Chip z.B. ein RAM
// A0,A1 adressieren die gewnschte Position auf dem Display
// A0=0,A1=0 ist ganz rechts 
//
// /CU wozu dieser Pin gut ist wei ich nicht. Wenn jemand ne
// Ahnung hat was man damit anfangen bitte unbedingt bei mir melden !
//
// Der Zeichensatz vom DL1416 geht nur von 0x20-0x5F
// Das sind die Zahlen 0-9, alle Grobuchstaben und
// ein paar andere Zeichen wie +,-,*,? usw.
// Kleinbuchstaben sind nicht enthalten !
// Die Zeichencodes entsprechen dem ASCII-Zeichensatz
//
// Schreibt man Zeichencodes < 0x20 oder > 0x5F ins
// Display, wird die entsprechende Stelle gelscht.
//
// hk@holger-klabunde.de
// www.holger-klabunde.de
// 14.03.2002
// Compiler CC5x

#include "prozess.h"      /* Der verwendete Prozessor */

#include "dl1416.h"
#include "protos.h"

unsigned char ledport; //Register zum merken letzter Zustand PORTB
unsigned char dlpos;   //aktuelle Position auf dem Display, 1 = ganz links

//*******************************************
// Schreibt ein Datenbyte in die LED-Anzeige
//*******************************************
void DLWriteByte(unsigned char data)
{
 unsigned char buff;

 switch(dlpos)
  {
   case 1: DLA0=1; DLA1=1; break; //ganz links
   case 2: DLA0=0; DLA1=1; break;
   case 3: DLA0=1; DLA1=0; break;
   case 4: DLA0=0; DLA1=0; break; //ganz rechts
   default: break;
  }

 ledport=PORTB;   //alten Wert merken, besonders A0
 buff=data<<1;    //D0..6 auf RB1..7 schieben

 ledport&=~0xFE;  //RB1..7 lschen
 ledport|=buff;   //zum reinodern von D0..6
 PORTB=ledport;   //Daten ausgeben

//Daten schreiben CE=0, WR=0
 DLCE=0;
 DLWR=0;

 DLWR=1;
 DLCE=1;

 dlpos++;             //Cursor automatisch bewegen
 if(dlpos>4 || dlpos==0) dlpos=1; //ganz rechts angekommen, also links wieder anfangen
}

//**************************************************
//Schreibt einen Text in die LED-Anzeige
//**************************************************
void DLWrite(const char *__putstr)
{
 char __ps;
 
 __ps = *__putstr;
 
  while(__ps>0)
   {
    __putstr++;
    if (__ps== 0) break;
    DLWriteByte(__ps);
    __ps = *__putstr;
   }
}

//****************************************************
// Setzt Position x auf der LED-Anzeige 
// 1 ist ganz links
//****************************************************
void DLPos(unsigned char position)
{
 dlpos=position;
 if(dlpos==0 || dlpos>4) dlpos=1;
}

//*************************************************
// Initialisiert die LED-Anzeige
// Cursor steht dann auf Position 1 ganz links
//*************************************************
void DLCls()
{
 int i;

 DLWR=1;
 DLCE=1;

 dlpos=1;  //Position ganz links

//Ich wei mangels Datenblatt nicht warum man dies hier tun mu,
//aber ohne luft das Display nicht ! Keine Ahnung wofr /CU
//benutzt wird ???????
//Alle 4 Stellen mit 0x20 beschreiben, vorher /CU=0, danach /CU=1 
//Ermittelt durch willenloses ausprobieren :(
 for(i=0; i<4; i++)
  {
   DLCU=0;
   DLWriteByte(0x20);
   DLCU=1;
  }

 dlpos=1;   //Position ganz links
}

//Zeigt ein Byte im HexCode an
void DLShowHex(unsigned char by)
{
 unsigned char buff;

 buff=by>>4; //Highnibble zuerst
 if(buff<10) buff+=0x30; //ASCII Code Zahlen
 else buff+=0x37;        //Grobuchstaben
 DLWriteByte(buff);

 buff=by&0x0f; //Danach das Lownibble
 if(buff<10) buff+=0x30; //ASCII Code Zahlen
 else buff+=0x37;        //Grobuchstaben
 DLWriteByte(buff);
}
